-- navmapui

-- Copyright (c) 2008 Stefan Scheffler

-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:

-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.

-- TODO: instead of updating all windows every time it could do it on *_SHOW events

local navroute_controls = {}
local navroutes = {}
local route_names = {}

local function load_routes()
	local num = gkini.ReadInt("navmapui", "nroutes", 0)
	for i=1,num do
		local route = unspickle(gkini.ReadString("navmapui", tostring(i), ""))
		if route.name then
			navroutes[route.name] = route.opt
		end
	end
end

local function save_routes()
	local i = 0
	for route,opt in pairs(navroutes) do
		i = i + 1
		gkini.WriteString("navmapui", tostring(i), spickle({name=route, opt=opt}))
	end
	gkini.WriteInt("navmapui", "nroutes", i)
end

-- small wrapper AbbrLocationStr
-- it doesn't like to be called without args
local function abbr_location(sector)
	if sector then 
		return AbbrLocationStr(sector)
	else
		return "?"
	end	
end

-- update state of added buttons and the entry
-- route is the string to put into the entry
local function update_controls(route)
	for _,controls in ipairs(navroute_controls) do
		if route and #route > 0 then
			controls.entry.value = route
			controls.plot.active = "YES"
			controls.add.active = "YES"
			if navroutes[route] then
				controls.del.active = "YES"
				controls.plot.active = "YES"
			else
				controls.del.active = "NO"
				controls.plot.active = "NO"
			end
		else
			controls.entry.value = route
			controls.add.active = "NO"
			controls.del.active = "NO"
			controls.plot.active = "NO"
		end
	end
end

local function update_lists()
	route_names = {}
	for _,controls in ipairs(navroute_controls) do
		local names = {}
		for i in pairs(navroutes) do
			table.insert(names, i)
		end
		table.sort(names)
		
		for i,v in ipairs(names) do
			local route = navroutes[v]

			controls.list[i] = v:sub(1, 10).." ("..
				           abbr_location(route.start)
					   .."->"..
					   abbr_location(route.dest)
					   ..")"
		end
		controls.list[#names+1] = nil

		route_names = names
	end
end


local function create_route_table()
	return {start=GetCurrentSectorid(), 
		first=NavRoute.GetFinalDestination(),
		dest=NavRoute.GetFinalDestination()}
end

-- adds the ui elements to a nav window
local function add_elements(navtab)
	local gnc = iup.GetNextChild
	local gb = iup.GetBrother
	
	-- could use one a few levels higher too but that makes aligning harder
	local box = gnc(gb(gnc(gb(gb(gnc(navtab))))))

	local list = iup.list{dropdown="YES", expand="HORIZONTAL", size=1}
	local addbutton = iup.stationbutton{title="Save", expand="HORIZONTAL", active="NO"}
	local delbutton = iup.stationbutton{title="Del", expand="HORIZONTAL", active="NO"}
	local plotbutton = iup.stationbutton{title="Plot", expand="HORIZONTAL", active="NO"}

	local entry = iup.text{expand="HORIZONTAL"}

	box:append(iup.vbox{iup.fill{size=5},
			    list, 
			    entry,
			    iup.hbox{plotbutton,
				     addbutton,
				     delbutton}})

	function list:action(value, index, select)
		local name = route_names[index]
		if select == 1 and name then
			
			NavRoute.load(name)
			entry.value = name
			plotbutton.active = "YES"
			addbutton.active = "YES"
			delbutton.active = "YES"
		end
	end

	function addbutton:action()
		if entry.value and #entry.value > 0 then
			NavRoute.save(NavRoute.GetCurrentRoute(), entry.value)
			navroutes[entry.value] = create_route_table()
			save_routes()
			update_lists()
			update_controls(entry.value)
		end
	end

	function delbutton:action()
		if entry.value and #entry.value > 0 then
			navroutes[entry.value] = nil
			entry.value = ""
			save_routes()
			update_lists()
			update_controls()
		end	
	end

	function plotbutton:action()
		if entry.value and #entry.value > 0 then
			NavRoute.load(entry.value)
		end
	end

	function entry:action(char, value)
		update_controls(value)
	end

	table.insert(navroute_controls, {list=list, add=addbutton, del=delbutton, plot=plotbutton, entry=entry})
end

add_elements(StationPDAShipNavigationTab)
add_elements(CapShipPDAShipNavigationTab)
add_elements(PDAShipNavigationTab)

-- crude hack to get list of routes
-- tell game to print them out and read the printed lines
local function sync_routes()
	local TERMINATOR = "end navmap sync"
	
	local navroute_names = {}

	navroutes = {}

	local msg = {}
	function msg:OnEvent(name, data)
		if data.msg:sub(1,1) == "\t" then
			navroute_names[data.msg:sub(2)] = true
		end
		if data.msg == TERMINATOR then
			UnregisterEvent(self, "CHAT_MSG_PRINT")

			for route in pairs(navroute_names) do
				NavRoute.load(route)
				navroutes[route] = create_route_table()
				-- curent sector as starting point is wrong in this case
				navroutes[route].start = nil
			end

			update_lists()
			save_routes()
		end
	end
	
	RegisterEvent(msg, "CHAT_MSG_PRINT")
	NavRoute.list()
	ProcessEvent("CHAT_MSG_PRINT", {msg=TERMINATOR})
end
RegisterUserCommand("navmapui_sync", sync_routes)

-- look for new routes
local navmapui = {}
function navmapui:OnEvent(name, data)
	if name == "NAVROUTE_SAVE" then
		if #NavRoute.GetCurrentRoute() > 0 then 
			navroutes[data] = create_route_table()
			save_routes()
			update_lists()
		end
	end
end
RegisterEvent(navmapui, "NAVROUTE_SAVE")

spuck = spuck or {}
spuck.navmapui = navmapui

load_routes()
update_lists()
